Attribute VB_Name = "modBitmap"
Option Explicit


'========================================================================================================
'
' modBitmap Module
' ----------------
'
' Created By  : Kevin Wilson
'               http://w3.one.net/~kevinw/VB   ( The VB Zone )
'
' Last Update : January 27, 2000
'
' VB Versions : 5.0 / 6.0
'
' Requires    : NOTHING
'
' Description : This module is a collection of very useful functions designed for use with Graphics
'               manipulation.  You can use it to find out information about pictures in memory, or to
'               render such pictures on to specified Device Contexts (DC).
'
'               If any of the functions in this module fail, you can most likely get extended error
'               information by calling the "GetLastError" Win32 API function.
'
' Example Use :
'
'  Dim TheHeight As Long
'  Dim TheWidth  As Long
'
'  ' Show the form so you can see the drawing as it happens in debug mode (step by step)
'  Me.Show
'
'  ' Make the picture redraw itself so it doesn't go away if the form loses focus
'  Picture1.AutoRedraw = True
'
'  ' Get the height/width in pixels
'  If Convert_HM_PX(Me.Picture.Height, Me.Picture.Width, TheHeight, TheWidth, True) = True Then
'
'    ' Draw the picture onto the PictureBox object called "Picture1" and
'    ' invert the colors by specifying "SCRINVERT" as the raster operation
'    ' instead of the default "SCRCOPY"
'    If RenderPicture(Picture1.hdc, , Me.Picture.Handle, 0, 0, TheHeight, TheWidth, SRCINVERT, , , , True, Picture1.hwnd) = True Then
'      Debug.Print "SUCCESS!"
'    Else
'      Debug.Print "FAILED!"
'    End If
'  End If
'
'  ' Set the "Picture" property to equal the "Image" property
'  Set Picture1.Picture = Picture1.Image
'
'========================================================================================================


' Type - General
Public Type RECT
  Left   As Long
  Top    As Long
  Right  As Long
  Bottom As Long
End Type

' Type - General
Public Type POINTAPI
  X As Long
  Y As Long
End Type

' Type - OleCreatePictureIndirect
Public Type GUID
  Data1    As Long
  Data2    As Integer
  Data3    As Integer
  Data4(7) As Byte
End Type

' Type - OleCreatePictureIndirect
Public Type PicBmp
  Size     As Long
  Type     As Long
  hBmp     As Long
  hPal     As Long
  reserved As Long
End Type

' Type - GetObjectAPI.lpObject
Public Type BITMAP
  bmType       As Long
  bmWidth      As Long
  bmHeight     As Long
  bmWidthBytes As Long
  bmPlanes     As Integer
  bmBitsPixel  As Integer
  bmBits       As Long
End Type

' Constants - BitBlt.dwRop
Public Enum RasterOperations
  SRCCOPY = &HCC0020          ' Copies the source rectangle directly to the destination rectangle.
  SRCPAINT = &HEE0086         ' Combines the colors of the source and destination rectangles by using the Boolean OR operator.
  SRCAND = &H8800C6           ' Combines the colors of the source and destination rectangles by using the Boolean AND operator.
  SRCINVERT = &H660046        ' Combines the colors of the source and destination rectangles by using the Boolean XOR operator.
  SRCERASE = &H440328         ' Combines the inverted colors of the destination rectangle with the colors of the source rectangle by using the Boolean AND operator.
  NOTSRCCOPY = &H330008       ' Copies the inverted source rectangle to the destination.
  NOTSRCERASE = &H1100A6      ' Combines the colors of the source and destination rectangles by using the Boolean OR operator and then inverts the resultant color.
  MERGECOPY = &HC000CA        ' Merges the colors of the source rectangle with the brush currently selected in hdcDest, by using the Boolean AND operator.
  MERGEPAINT = &HBB0226       ' Merges the colors of the inverted source rectangle with the colors of the destination rectangle by using the Boolean OR operator.
  PATCOPY = &HF00021          ' Copies the brush currently selected in hdcDest, into the destination bitmap.
  PATPAINT = &HFB0A09         ' Combines the colors of the brush currently selected in hdcDest, with the colors of the inverted source rectangle by using the Boolean OR operator. The result of this operation is combined with the colors of the destination rectangle by using the Boolean OR operator.
  PATINVERT = &H5A0049        ' Combines the colors of the brush currently selected in hdcDest, with the colors of the destination rectangle by using the Boolean XOR operator.
  DSTINVERT = &H550009        ' Inverts the destination rectangle.
  BLACKNESS = &H42            ' Fills the destination rectangle using the color associated with index 0 in the physical palette. (This color is black for the default physical palette.)
  WHITENESS = &HFF0062        ' Fills the destination rectangle using the color associated with index 1 in the physical palette. (This color is white for the default physical palette.)
  NOMIRRORBITMAP = &H80000000 ' Windows 98, Windows 2000: Prevents the bitmap from being mirrored.
  CAPTUREBLT = &H40000000     ' Windows 98, Windows 2000: Includes any windows that are layered on top of your window in the resulting image. By default, the image only contains your window.
End Enum

' Constants - LoadResData
Public Enum ResTypes
  rt_Bitmap = vbResBitmap
  rt_Icon = vbResIcon
  rt_Cursor = vbResCursor
  rt_Custom = 3
End Enum

' Constants - General
Public Const MAX_PATH = 260

' Constants - GetDeviceCaps.nIndex
Public Const HORZSIZE = 4    ' Width, in millimeters, of the physical screen.
Public Const VERTSIZE = 6    ' Height, in millimeters, of the physical screen.
Public Const HORZRES = 8     ' Width, in pixels, of the screen.
Public Const VERTRES = 10    ' Height, in raster lines, of the screen.
Public Const BITSPIXEL = 12  ' Number of adjacent color bits for each pixel.

' Constants - OleCreateBitmapIndiect (Return Values)
Public Const S_OK = 0                   ' The new picture object was created successfully.
Public Const E_NOINTERFACE = &H80004002 ' The object does not support the interface specified in riid.
Public Const E_POINTER = &H80004003     ' The address in pPictDesc or ppvObj is not valid. For example, it may be NULL.
Public Const E_INVALIDARG = &H80000003  ' One or more arguments are invalid
Public Const E_OUTOFMEMORY = &H8007000E ' Ran out of memory
Public Const E_UNEXPECTED = &H8000FFFF  ' Catastrophic failure

' Constants - GetCurrentObject.uObjectType
Public Const OBJ_BITMAP = 7      ' Returns the current selected bitmap
Public Const OBJ_BRUSH = 2       ' Returns the current selected brush
Public Const OBJ_COLORSPACE = 14 ' Returns the current color space
Public Const OBJ_FONT = 6        ' Returns the current selected font
Public Const OBJ_PAL = 5         ' Returns the current selected pal
Public Const OBJ_PEN = 1         ' Returns the current selected pen

' Constants - BITMAP.bmType & CopyImage.uType
Public Const IMAGE_BITMAP = 0
Public Const IMAGE_CURSOR = 1
Public Const IMAGE_ICON = 2
Public Const IMAGE_ENHMETAFILE = 3

' Constants - CopyImage.fuFlags
Public Const LR_COPYDELETEORG = &H8       ' Deletes the original image after creating the copy.
Public Const LR_COPYFROMRESOURCE = &H4000 ' Tries to reload an icon or cursor resource from the original resource file rather than simply copying the current image. This is useful for creating a different-sized copy when the resource file contains multiple sizes of the resource. Without this flag, CopyImage stretches the original image to the new size. If this flag is set, CopyImage uses the size in the resource file closest to the desired size.  This will succeed only if hImage was loaded by LoadIcon or LoadCursor, or by LoadImage with the LR_SHARED flag.
Public Const LR_COPYRETURNORG = &H4       ' Returns the original hImage if it satisfies the criteria for the copythat is, correct dimensions and color depthin which case the LR_COPYDELETEORG flag is ignored. If this flag is not specified, a new object is always created.
Public Const LR_CREATEDIBSECTION = &H2000 ' If this is set and a new bitmap is created, the bitmap is created as a DIB section. Otherwise, the bitmap image is created as a device-dependent bitmap. This flag is only valid if uType is IMAGE_BITMAP.
Public Const LR_MONOCHROME = &H1          ' Creates a new monochrome image.

' Constants - RedrawWindow.fuRedraw
Public Const RDW_ERASE = &H4
Public Const RDW_FRAME = &H400
Public Const RDW_INTERNALPAINT = &H2
Public Const RDW_INVALIDATE = &H1
Public Const RDW_NOERASE = &H20
Public Const RDW_NOFRAME = &H800
Public Const RDW_NOINTERNALPAINT = &H10
Public Const RDW_VALIDATE = &H8
Public Const RDW_ERASENOW = &H200
Public Const RDW_UPDATENOW = &H100
Public Const RDW_ALLCHILDREN = &H80
Public Const RDW_NOCHILDREN = &H40

' Win32 Function Declarations
Public Declare Function BitBlt Lib "GDI32" (ByVal hDC_Destination As Long, ByVal X_Dest As Long, ByVal Y_Dest As Long, ByVal Width_Dest As Long, ByVal Height_Dest As Long, ByVal hDC_Source As Long, ByVal X_Src As Long, ByVal Y_Src As Long, ByVal RasterOperation As Long) As Long
Public Declare Function CopyImage Lib "user32" (ByVal hImage As Long, ByVal uType As Long, ByVal OutputWidth As Long, ByVal OutputHeight As Long, ByVal fuFlags As Long) As Long
Public Declare Function CreateCompatibleBitmap Lib "GDI32" (ByVal hDC As Long, ByVal nWidth As Long, ByVal nHeight As Long) As Long
Public Declare Function CreateCompatibleDC Lib "GDI32" (ByVal hDC As Long) As Long
Public Declare Function DeleteDC Lib "GDI32" (ByVal hDC As Long) As Long
Public Declare Function DeleteObject Lib "GDI32" (ByVal hGDIObj As Long) As Long
Public Declare Function GetCurrentObject Lib "GDI32" (ByVal hDC As Long, ByVal uObjectType As Long) As Long
Public Declare Function GetDC Lib "user32" (ByVal hwnd As Long) As Long
Public Declare Function GetDesktopWindow Lib "user32" () As Long
Public Declare Function GetDeviceCaps Lib "GDI32" (ByVal hDC As Long, ByVal nIndex As Long) As Long
Public Declare Function GetObjectAPI Lib "GDI32" Alias "GetObjectA" (ByVal hObject As Long, ByVal nCount As Long, lpObject As Any) As Long
Public Declare Function OleCreatePictureIndirect Lib "OLEPRO32.DLL" (ByRef PicDesc As PicBmp, ByRef RefIID As GUID, ByVal fPictureOwnsHandle As Long, ByRef IPic As StdPicture) As Long 'As IPicture) As Long
Public Declare Function OleTranslateColor Lib "OLEPRO32.DLL" (ByVal OLE_COLOR As Long, ByVal hPalette As Long, pColorRef As Long) As Long
Public Declare Function RedrawWindow Lib "user32" (ByVal hwnd As Long, lprcUpdate As Any, ByVal hrgnUpdate As Long, ByVal fuRedraw As Long) As Long
Public Declare Function ReleaseDC Lib "user32" (ByVal hwnd As Long, ByVal hDC As Long) As Long
Public Declare Function SelectObject Lib "GDI32" (ByVal hDC As Long, ByVal hGDIObj As Long) As Long
Public Declare Function SetBkColor Lib "GDI32" (ByVal hDC As Long, ByVal crColor As Long) As Long
Public Declare Function StretchBlt Lib "GDI32" (ByVal hDC_Destination As Long, ByVal X_Dest As Long, ByVal Y_Dest As Long, ByVal New_Width As Long, ByVal New_Height As Long, ByVal hDC_Source As Long, ByVal X_Src As Long, ByVal Y_Src As Long, ByVal Orig_Width As Long, ByVal Orig_Height As Long, ByVal RasterOperation As Long) As Long


'========================================================================================================
'
' Convert_HM_PX
'
' When dealing with the "Picture" property of VB objects such as PictureBox or Form, or StdPicture
' objects, the Height & Width properties of such is not measured in Pixels or Twips... but in something
' called "HiMetric".  This function takes the height and width measurements of a picture in HiMetric
' and converts it to Pixels so that it can be used with standard Win32 API calls, or with VB objects
' that have their "ScaleMode" property set to "vbPixels"
'
' NOTE - You can also use the "GetPictureInfo" function to get the height and/or width of a picture
' in pixels.
'
' Parameter:              Use:
' --------------------------------------------------
' InputHeight             Optional. Specifies the height of the picture in HiMetric
' InputWidth              Optional. Specifies the width of the picture in HiMetric
' OutputHeight            Optional. Returns the height of the picture in Pixels (if InputHeight is valid)
' OutputWidth             Optional. Returns the width of the picture in Pixels (if InputWidth is valid)
' VB_Picture              Optional. If set to TRUE, the calculation used to get the desired return value
'                         uses the Screen.TwipsPerPixel properties to get the TwipsPerPixel instead of
'                         using a more accurate calculation of TwipsPerPixel.  The return value is correct
'                         for use with the Picture property of VB objects like PictureBox, & StdPicture.
'                         If set to FALSE, the calculation used to get the desired return value uses a
'                         calculation to get and use a more accurate measurement of the TwipsPerPixel.
'                         This is more accurate for use with Win32 API calls.
'
' Return:
' -------
' If the function succeeds, the return is TRUE
' If the function fails, the return is FALSE
'
' --------------------------------------------------
' These type definitions were taken from OCIDL.H
' --------------------------------------------------
' typedef LONG OLE_XPOS_HIMETRIC;
' typedef LONG OLE_YPOS_HIMETRIC;
' typedef LONG OLE_XSIZE_HIMETRIC;
' typedef LONG OLE_YSIZE_HIMETRIC;
'
'========================================================================================================
Public Function Convert_HM_PX(Optional ByVal InputHeight As Long, _
                              Optional ByVal InputWidth As Long, _
                              Optional ByRef OutputHeight As Long, _
                              Optional ByRef OutputWidth As Long, _
                              Optional ByVal VB_Picture As Boolean = True) As Boolean
On Error Resume Next
  
  Dim TwipsX    As Single
  Dim TwipsY    As Single
  
  ' Reset the return values
  OutputHeight = 0
  OutputWidth = 0
  
  ' Make sure the parameters passed are valid
  If InputHeight = 0 And InputWidth = 0 Then Exit Function
  
  ' If the user specifies to do the convertion for a Visual Basic Picture, use the
  ' "Screen" object to get the approximate TwipsPerPixel
  If VB_Picture = True Then
    OutputWidth = CLng(((InputWidth / 2540) * 1440) / Screen.TwipsPerPixelX)
    OutputHeight = CLng(((InputHeight / 2540) * 1440) / Screen.TwipsPerPixelY)
    
  ' If the user doesn't specify to do the convertion for a Visual Basic Picture, assume
  ' it's for a Win32 API call and calculate the exact TwipsPerPixel to be more accurate
  Else
    If GetDisplayInfo(, , TwipsX, TwipsY) = False Then Exit Function
    OutputWidth = CLng((InputWidth / 2540 * 1440) / TwipsX)
    OutputHeight = CLng((InputHeight / 2540 * 1440) / TwipsY)
  End If
  
  ' Function succeeded
  Convert_HM_PX = True
  
End Function

'========================================================================================================
'
' Convert_PX_HM
'
' When dealing with the "Picture" property of VB objects such as PictureBox or Form, or StdPicture
' objects, the Height & Width properties of such is not measured in Pixels or Twips... but in something
' called "HiMetric".  This function takes the height and width measurements of a picture in Pixels
' and converts it to HiMetric so that it can be used with VB calls, etc.
'
' NOTE - When the "VB_Picture" parameter is set to FALSE, the return values of this function are VERY
' close, but not exact because of how the calculations and number rounding works.  To see this effect,
' use the Convert_HM_PX function to take the height/width of a picture and convert them to pixels...
' then take the return values from that and use this function to convert them back to their original
' HiMetrics measurement.  The results will be very close, but not exact.  This shouldn't be a problem
' because I would think it would be a rare thing that you'd want to convert Pixels to HiMetric (I even
' considered leaving this function out of the module).
'
' Parameter:              Use:
' --------------------------------------------------
' InputHeight             Optional. Specifies the height of the picture in Pixels
' InputWidth              Optional. Specifies the width of the picture in Pixels
' OutputHeight            Optional. Returns the height of the picture in HiMetric (if InputHeight is valid)
' OutputWidth             Optional. Returns the width of the picture in HiMetric (if InputWidth is valid)
' VB_Picture              Optional. If set to TRUE, the calculation used to get the desired return value
'                         uses the Screen.TwipsPerPixel properties to get the TwipsPerPixel instead of
'                         using a more accurate calculation of TwipsPerPixel.  The return value is correct
'                         for use with the Picture property of VB objects like PictureBox, & StdPicture.
'                         If set to FALSE, the calculation used to get the desired return value uses a
'                         calculation to get and use a more accurate measurement of the TwipsPerPixel.
'                         This is more accurate for use with Win32 API calls.
'
' Return:
' -------
' If the function succeeds, the return is TRUE
' If the function fails, the return is FALSE
'
' --------------------------------------------------
' These type definitions were taken from OCIDL.H
' --------------------------------------------------
' typedef LONG OLE_XPOS_HIMETRIC;
' typedef LONG OLE_YPOS_HIMETRIC;
' typedef LONG OLE_XSIZE_HIMETRIC;
' typedef LONG OLE_YSIZE_HIMETRIC;
'
'========================================================================================================
Public Function Convert_PX_HM(ByVal InputHeight As Long, _
                              ByVal InputWidth As Long, _
                              ByRef OutputHeight As Long, _
                              ByRef OutputWidth As Long, _
                              Optional ByVal VB_Picture As Boolean = True) As Boolean
On Error Resume Next
  
  Dim TwipsX    As Single
  Dim TwipsY    As Single
  
  ' Reset the return values
  OutputHeight = 0
  OutputWidth = 0
  
  ' Make sure the parameters passed are valid
  If InputHeight = 0 And InputWidth = 0 Then Exit Function
  
  ' If the user specifies to do the convertion for a Visual Basic Picture, use the
  ' "Screen" object to get the approximate TwipsPerPixel
  If VB_Picture = True Then
    OutputHeight = CLng(((InputHeight * Screen.TwipsPerPixelY) / 1440) * 2540)
    OutputWidth = CLng(((InputWidth * Screen.TwipsPerPixelX) / 1440) * 2540)
    
  ' If the user doesn't specify to do the convertion for a Visual Basic Picture, assume
  ' it's for a Win32 API call and calculate the exact TwipsPerPixel to be more accurate
  Else
    If GetDisplayInfo(, , TwipsX, TwipsY) = False Then Exit Function
    OutputHeight = CLng(((InputHeight * TwipsX) / 1440) * 2540)
    OutputWidth = CLng(((InputWidth * TwipsY) / 1440) * 2540)
  End If
  
  ' Function succeeded
  Convert_PX_HM = True
  
End Function

'========================================================================================================
'
' CopyPicture
'
' This function takes the handle to the picture passed in via the "IN_hPicture" parameter and makes a
' copy of it... returning it via the "OUT_hPicture" parameter.
'
' Parameter:              Use:
' --------------------------------------------------
' IN_hPicture             Specifies the handle to the picture to copy
' OUT_hPicture            Returns the newly created copy of the original picture
'
' Return:
' -------
' If the function succeeds, the return is TRUE
' If the function fails, the return is FALSE
'
'========================================================================================================
Public Function CopyPicture(ByVal IN_hPicture As Long, _
                            ByRef OUT_hPicture As Long) As Boolean
  
  Dim TheHeight As Long
  Dim TheWidth  As Long
  Dim TheType   As Long
  
  ' Clear the return variable
  OUT_hPicture = 0
  
  ' Make sure parameters passed are valid
  If IN_hPicture = 0 Then Exit Function
  
  ' Get the dimentions and type of picture to copy
  If GetPictureInfo(IN_hPicture, TheHeight, TheWidth, TheType) = False Then Exit Function
  
  ' Copy the image
  OUT_hPicture = CopyImage(IN_hPicture, TheType, TheWidth, TheHeight, 0)
  If OUT_hPicture <> 0 Then CopyPicture = True
  
End Function

'========================================================================================================
'
' CreateMask
'
' This function takes the specified picture and creates a sprite and a mask from it.  The sprite is the
' same as the original picture, but the color that is specified by the "TransparentColor" parameter is
' changed to WHITE (this serves to designate where the transparency will be).  The mask is a black
' silhouette of the original picture with a white background.
'
' When the mask is combined with another picture using the Win32 "BitBlt" API with the "MERGEPAINT"
' raster operation, it puts a white silhouette of the original picture (without the transparent region).
' When the sprite is combined with the picture that the mask was combined with in the same location
' as the mask using the Win32 "BitBlt" API with the "SRCAND" raster operation, the original picture is
' displayed on the picture as a transparent picture (the specified background color, or transparent
' color no longer shows up.
'
' 
' Dim PicBG      As StdPicture
' Dim PicImg     As StdPicture
' Dim hSpriteDC  As Long
' Dim hMaskDC    As Long
' Dim hSpriteBMP As Long
' Dim hMaskBMP   As Long
' Dim TheWidth   As Long
' Dim TheHeight  As Long
'
' Me.Show
' Set PicBG = LoadPicture("C:\Background.bmp")
' Set PicImg = LoadPicture("C:\Image.bmp")
' RenderPicture Me.hDC, , PicBG.Handle
' GetPictureInfo PicImg.Handle, TheHeight, TheWidth
' CreateMask PicImg.Handle, CLng("&H00FF00"), hSpriteDC, hMaskDC, hSpriteBMP, hMaskBMP
' BitBlt Me.hDC, 0, 0, TheWidth, TheHeight, hMaskDC, 0, 0, MERGEPAINT
' BitBlt Me.hDC, 0, 0, TheWidth, TheHeight, hSpriteDC, 0, 0, SRCAND
' MemoryDC_Delete hSpriteDC, hSpriteBMP
' MemoryDC_Delete hMaskDC, hMaskBMP
' Set PicBG = Nothing
' Set PicImg = Nothing
'
' 
'
' Parameter:              Use:
' --------------------------------------------------
' hPicture                Handle to the picture to create the sprite and mask from
' TransparentColor        Specifies the color that is to be made transparent (background color)
' Return_SpriteDC         Optional. Returns the handle to the DC created that contains the sprite
' Return_MaskDC           Optional. Returns the handle to the DC created that contains the mask
' Return_SpritePrevBMP    Optional. Returns the handle to the picture that was previously in the sprite DC.
'                         This is important because you should always select the OLD picture back into
'                         a DC before deleting it.
' Return_MaskPrevBMP      Optional. Returns the handle to the picture that was previously in the mask DC.
'                         This is important because you should always select the old picture back into
'                         a DC before deleting it.
'
' Return:
' -------
' If the function succeeds, the return is TRUE
' If the function fails, the return is FALSE
'
'========================================================================================================
Public Function CreateMask(ByVal hPicture As Long, _
                           ByVal TransparentColor As Long, _
                           Optional ByRef Return_SpriteDC As Long = -1, _
                           Optional ByRef Return_MaskDC As Long = -1, _
                           Optional ByRef Return_SpritePrevBMP As Long = -1, _
                           Optional ByRef Return_MaskPrevBMP As Long = -1) As Boolean
  
  Dim hScreenDC     As Long
  Dim PicHeight     As Long
  Dim PicWidth      As Long
  Dim hDC_Temp      As Long
  Dim hDC_Sprite    As Long
  Dim hDC_Mask      As Long
  Dim hPrev_Temp    As Long
  Dim hPrev_Sprite  As Long
  Dim hPrev_Mask    As Long
  Dim PreviousColor As Long
  Dim hMonoBitmap   As Long
  Dim hColrBitmap   As Long
  
  ' Make sure a valid picture was passed to use to get the mask
  If hPicture = 0 Then Exit Function
    
  ' If the user hasn't specified to return any values, exit
  If Return_SpriteDC = -1 And Return_MaskDC = -1 Then Exit Function
  
  ' Get the transparent color as a non-OLE color
  TransparentColor = TranslateColor(TransparentColor)
  If TransparentColor = -1 Then Exit Function
  
  ' Get the height & width of the picture
  If GetPictureInfo(hPicture, PicHeight, PicWidth) = False Then Exit Function
  
  ' Create the Device Contexts (DC) to work with
  hDC_Temp = MemoryDC_Create
  hDC_Mask = MemoryDC_Create
  hDC_Sprite = MemoryDC_Create
  
  ' Create a monochrome bitmap to create the mask from
  hMonoBitmap = CreateCompatibleBitmap(hDC_Temp, PicWidth, PicHeight)
  
  ' Create a color bitmap to select into the mask
  hScreenDC = GetDC(GetDesktopWindow)
  hColrBitmap = CreateCompatibleBitmap(hScreenDC, PicWidth, PicHeight)
  ReleaseDC GetDesktopWindow, hScreenDC
  
  ' Copy the sprite picture into the sprite DC making it color, then
  ' copy the monochrome bitmaps into the temp and mask DC's black & white
  LoadPictureToDC hDC_Temp, hMonoBitmap, hPrev_Temp
  LoadPictureToDC hDC_Mask, hColrBitmap, hPrev_Mask
  LoadPictureToDC hDC_Sprite, hPicture, hPrev_Sprite
  
  ' Set the background color to the transparent color.
  ' This will make the transparent color black when the mask is created
  PreviousColor = SetBkColor(hDC_Sprite, TransparentColor)
  
  ' Copy the sprite picture to the temp DC - This makes a copy of the
  ' sprite, but the copy is black and the transparent color is now transparent
  BitBlt hDC_Temp, 0, 0, PicWidth, PicHeight, hDC_Sprite, 0, 0, SRCCOPY
  
  ' Set the background color back to the original color
  SetBkColor hDC_Sprite, PreviousColor
  
  ' Copy the temp DC to the sprite - this makes the sprite's transparent
  ' color now show up as WHITE
  BitBlt hDC_Sprite, 0, 0, PicWidth, PicHeight, hDC_Temp, 0, 0, SRCPAINT
  
  ' Create the mask (Silhouette of the original on a white background)
  BitBlt hDC_Mask, 0, 0, PicWidth, PicHeight, hDC_Sprite, 0, 0, SRCCOPY
  BitBlt hDC_Mask, 0, 0, PicWidth, PicHeight, hDC_Temp, 0, 0, SRCCOPY
  
  ' Return the results and clean up extra memory
  CreateMask = True
  
  ' If the user specified to return the sprite, do so... else delete it
  If Return_SpriteDC = -1 Then
    MemoryDC_Delete hDC_Sprite, hPrev_Sprite
    Return_SpriteDC = 0
    Return_SpritePrevBMP = 0
  Else
    Return_SpriteDC = hDC_Sprite
    If Return_SpritePrevBMP = -1 Then
      DeleteObject hPrev_Sprite
    Else
      Return_SpritePrevBMP = hPrev_Sprite
    End If
  End If
  
  ' If the user specified to return the mask, do so... else delete it
  If Return_MaskDC = -1 Then
    MemoryDC_Delete hDC_Mask, hPrev_Mask
    Return_MaskDC = 0
    Return_MaskPrevBMP = 0
  Else
    Return_MaskDC = hDC_Mask
    If Return_MaskPrevBMP = -1 Then
      DeleteObject hPrev_Mask
    Else
      Return_MaskPrevBMP = hPrev_Mask
    End If
  End If
  
  ' Clean up memory used to create the sprite and mask
  DeleteObject hMonoBitmap
  DeleteObject hColrBitmap
  If hDC_Temp <> 0 Then MemoryDC_Delete hDC_Temp, hPrev_Temp
  
End Function

'========================================================================================================
'
' CreateOlePicture
'
' This function takes the handle to a picture and creates an OLE StdPicture object from it that can be
' used like the "Picture" properties of such VB objects as Forms, PictureBoxs, ImageBoxs, etc.
'
' Parameter:              Use:
' --------------------------------------------------
' PictureHandle           Handle to the picture to create
' PictureType             Optional. Specifies the type of picture object to create.  These are the
'                         different types of pictures that can be specified:
'                           vbPicTypeBitmap     <-- DEFAULT
'                           vbPicTypeEMetafile
'                           vbPicTypeIcon
'                           vbPicTypeMetafile
'                           vbPicTypeNone
' hPalette                Optional. Specifies the handle to a Palette to use in the createion process.
' Return_ErrNum           Optional. If an error occurs, the error number will be returned here.
' Return_ErrDesc          Optional. If an error occurs, the error description will be returned here.
'
' Return:
' -------
' If the function succeeds, the return is TRUE
' If the function fails, the return is FALSE
'
'========================================================================================================
Public Function CreateOlePicture(ByVal PictureHandle As Long, _
                                 Optional ByVal PictureType As PictureTypeConstants = vbPicTypeBitmap, _
                                 Optional ByVal hPalette As Long = 0, _
                                 Optional ByRef Return_ErrNum As Long, _
                                 Optional ByRef Return_ErrDesc As String) As StdPicture
On Error Resume Next
  
  Dim ReturnValue   As Long
  Dim PicInfo       As PicBmp
  Dim ThePicture    As StdPicture ' IPicture
  Dim IID_IDispatch As GUID
  
  ' Clear the return variables
  Return_ErrNum = 0
  Return_ErrDesc = ""
  
  ' Make sure the variable(s) passed are valid
  If PictureHandle = 0 Then
    Return_ErrNum = -1
    Return_ErrDesc = "Invalid bitmap handle"
  End If
  
  ' Fill GUID info
  With IID_IDispatch
    .Data1 = &H20400
    .Data4(0) = &HC0
    .Data4(7) = &H46
  End With
  
  ' Fill picture info
  With PicInfo
    .Size = Len(PicInfo)  ' Length of structure
    .Type = PictureType   ' Type of Picture
    .hBmp = PictureHandle ' Handle to bitmap
    .hPal = hPalette      ' Handle to palette (May be NULL)
  End With
  
  ' Create the picture
  ReturnValue = OleCreatePictureIndirect(PicInfo, IID_IDispatch, 1, ThePicture)
  If ReturnValue <> S_OK Then
    GoTo ErrorTrap
  End If
  
  ' Return the new picture
  Set CreateOlePicture = ThePicture
  
  Exit Function
  
ErrorTrap:
  
  Return_ErrNum = ReturnValue
  Select Case ReturnValue
    Case E_NOINTERFACE
      Return_ErrDesc = "The object does not support the interface specified in riid."
    Case E_POINTER
      Return_ErrDesc = "The address in pPictDesc or ppvObj is not valid. For example, it may be NULL."
    Case E_INVALIDARG
      Return_ErrDesc = "One or more arguments are invalid."
    Case E_OUTOFMEMORY
      Return_ErrDesc = "Ran out of memory."
    Case E_UNEXPECTED
      Return_ErrDesc = "Catastrophic Failure."
    Case Else
      Return_ErrDesc = "Unknown Error."
  End Select
  
End Function

'========================================================================================================
'
' ExtractFromRES
'
' This function extracts the specified files from a VB resource that is included in your project.
'
' Parameter:              Use:
' --------------------------------------------------
' OutputFile              Specifies the name of the file to extract the resource to.  If the file
'                         specified already exists, and the "OverwriteFile" parameter is set to TRUE,
'                         the file will be automatically overwritten.  If the "OverwriteFile" parameter
'                         is FALSE and the "ConfirmOnOverwrite" parameter is TRUE, the user is prompted
'                         to overwrite.  If the file exists, the OverwriteFile parameter is FALSE, and
'                         the ConfirmOnOverwrite parameter is FALSE, the function fails out.
'                         NOTE: If you specify "vbNullString" or a blank string ("") for this parameter
'                         and specify TRUE for the "ReturnPicRef" parameter, the file will not be saved
'                         out, but a reference to it will be returned via the "Return_Picture" parameter.
' RES_ID                  Specifies the resource ID of the file to extract.  The default ID is 101.
' RES_Section             Specifies the section of resource to extract from.  When you add a file to a
'                         VB resource, the default type is "CUSTOM".  You can of course change the
'                         type of the file added in the resource editor.  This parameter is only used
'                         if the "FileType" parameter is rt_Custom.
' FileType                Specifies if the file to extract is a Bitmap, Icon, Cursor, or other file.
' OverwriteFile           If the specified file already exists and this parameter is TRUE, the file is
'                         overwritten.
' ConfirmOnOverwrite      If the file exists, and OverwriteFile is FALSE, and this parameter is TRUE,
'                         the user is prompted to overwrite the file.
' ReturnPicRef            If set to TRUE, the "Return_Picture" parameter returns a reference to the
'                         picture specified.  If the FileType is set to rt_Custom, and this parameter
'                         is set to true, it is assumed that there is a picture stored int he custom
'                         resource and this function attempts to load it.
' Return_Picture          If the "ReturnPicRef" parameter is set to TRUE, this parameter returns a
'                         reference to the picture loaded.  If the FileType is set to rt_Custom, and
'                         the "ReturnPicRef" parameter is set to true, it is assumed that there is a
'                         picture stored int he custom resource and this function attempts to load it.
'
' Return:
' -------
' If the function succeeds, the return is TRUE
' If the function fails, the return is FALSE
'
'========================================================================================================
Public Function ExtractFromRES(ByVal OutputFile As String, _
                               ByVal RES_ID As Long, _
                               Optional ByVal RES_Section As String, _
                               Optional ByVal FileType As ResTypes = rt_Custom, _
                               Optional ByVal OverwriteFile As Boolean = True, _
                               Optional ByVal ConfirmOnOverwrite As Boolean = False, _
                               Optional ByVal ReturnPicRef As Boolean = False, _
                               Optional ByRef Return_Picture As StdPicture) As Boolean
On Error GoTo ErrorTrap
  
  Dim ResFile() As Byte
  Dim TestVar   As Variant
  Dim ResPic    As StdPicture
  Dim FileNum   As Long
  Dim MyAnswer  As VbMsgBoxResult
  Dim DelFile   As Boolean
  
  ' Make sure parameters are valid
  If RES_ID < 1 Or RES_ID > 32767 Then
    MsgBox "Resource ID is invalid.  Value must be between 1 and 32767.", vbOKOnly + vbExclamation, "  Invalid Resource ID"
    Exit Function
  ElseIf FileType = rt_Custom And RES_Section = "" Then
    MsgBox "No resource type specified.", vbOKOnly + vbExclamation, "  Error Extracting Resource"
    Exit Function
  ElseIf OutputFile = "" And ReturnPicRef = False Then
    MsgBox "No output file specified to extract resource to.", vbOKOnly + vbExclamation, "  No Output File Specified"
    Exit Function
  ElseIf Dir(OutputFile) <> "" And OverwriteFile = False Then
    If ConfirmOnOverwrite = True Then
      MyAnswer = MsgBox(OutputFile & Chr(13) & "This file already exists." & Chr(13) & Chr(13) & "Overwrite existing file?", vbYesNo + vbExclamation, "  Confirm File Overwrite")
      If MyAnswer = vbNo Then
        ExtractFromRES = True
        Exit Function
      End If
    Else
      Exit Function
    End If
  End If
  
  ' Extract and process the specified file
  Select Case FileType
    
    ' Save out the specified picture resource
    Case rt_Bitmap, rt_Cursor, rt_Icon
      Set ResPic = LoadResPicture(RES_ID, FileType)
      If ResPic Is Nothing Then Exit Function
      If OutputFile <> "" Then SavePicture ResPic, OutputFile
      If ReturnPicRef = True Then
        Set Return_Picture = ResPic
      Else
        Set ResPic = Nothing
      End If
      
    ' Save out the specified custom resource file
    Case rt_Custom
      ' If no output file specified, use a temporary one
      If OutputFile = "" Then
        OutputFile = "C:\TEMP.TMP"
        On Error Resume Next
        Kill OutputFile
        Err.Clear
        On Error GoTo ErrorTrap
        DelFile = True
      End If
      
      ' Load the resource
      ResFile = LoadResData(RES_ID, RES_Section)
      
      ' Check if an error occured while loading the resource
      On Error Resume Next
      Err.Clear
      TestVar = UBound(ResFile)
      If Err Then Err.Clear: Exit Function
      On Error GoTo ErrorTrap
      
      ' Save the resource file out to disk
      FileNum = FreeFile
      Open OutputFile For Binary As #FileNum
        Put #FileNum, 1, ResFile()
      Close #FileNum
      
      ' If the user specified to return the picture, assume the custom file is a picture
      ' and try to load it and return a reference to it
      If ReturnPicRef = True Then
        On Error Resume Next
        Set Return_Picture = LoadPicture(OutputFile)
        If Err Or (Return_Picture Is Nothing) Then
          Set Return_Picture = Nothing
          Err.Clear
        End If
      End If
      
      ' Clean up temp file
      If DelFile = True Then Kill OutputFile
  End Select
  
  ' Return SUCCESS
  ExtractFromRES = True
  
  Exit Function
  
ErrorTrap:
  
  If Err.Number = 0 Then      ' No Error
    Resume Next
  ElseIf Err.Number = 20 Then ' Resume Without Error
    Resume Next
  Else                        ' Unknown Error
    MsgBox Err.Source & " encountered the following error:" & Chr(13) & Chr(13) & "Error Number = " & CStr(Err.Number) & Chr(13) & "Error Description = " & Err.Description, vbOKOnly + vbExclamation, "  Error  -  " & Err.Description
    Err.Clear
  End If
  
End Function

'========================================================================================================
'
' GetDisplayInfo
'
' This function returns general information about the current screen display settings.
'
' Parameter:              Use:
' --------------------------------------------------
' ScreenWidth             Returns the width of the current display resolution in pixels
' ScreenHeight            Returns the height of the current display resolution in pixels
' TwipsX                  Returns the current number of twips per each pixel accross the X axis of the
'                         current screen display (see also Screen.TwipsPerPixelX)
' TwipsY                  Returns the current number of twips per each pixel accross the Y axis of the
'                         current screen display (see also Screen.TwipsPerPixelY)
' BitsPerPixel            Returns the current color depth of the screen display
'
' Return:
' -------
' If the function succeeds, the return is TRUE
' If the function fails, the return is FALSE
'
'========================================================================================================
Public Function GetDisplayInfo(Optional ByRef ScreenWidth As Long, _
                               Optional ByRef ScreenHeight As Long, _
                               Optional ByRef TwipsX As Single, _
                               Optional ByRef TwipsY As Single, _
                               Optional ByRef BitsPerPixel As Long) As Boolean
On Error Resume Next
  
  Dim TempW     As Long
  Dim TempH     As Long
  Dim hScreenDC As Long
  
  ' Reset the return values
  ScreenWidth = 0
  ScreenHeight = 0
  TwipsX = 0
  TwipsY = 0
  BitsPerPixel = 0
  
  ' Get the handle to the display area's Device Context
  hScreenDC = GetDC(GetDesktopWindow)
  If hScreenDC = 0 Then Exit Function
  
  ' Get the color depth
  BitsPerPixel = GetDeviceCaps(hScreenDC, BITSPIXEL)
  
  ' Get the screen width / height in pixels
  ScreenWidth = GetDeviceCaps(hScreenDC, HORZRES)
  ScreenHeight = GetDeviceCaps(hScreenDC, VERTRES)
  
  ' Get the physical width / height of the screen in millimeters
  TempH = GetDeviceCaps(hScreenDC, VERTSIZE)
  TempW = GetDeviceCaps(hScreenDC, HORZSIZE)
  
  ' Get the TwipsPerPixelX & TwipsPerPixelY (There are 56.7 twips per millimeter)
  TwipsX = CSng((56.7 * TempW) / ScreenWidth)
  TwipsY = CSng((56.7 * TempH) / ScreenHeight)
  
  ' Format the return to one decimal place
  TwipsX = CSng(Format(TwipsX, "0.0"))
  TwipsY = CSng(Format(TwipsY, "0.0"))
  
  ' Clean up and return
  ReleaseDC GetDesktopWindow, hScreenDC
  GetDisplayInfo = True
  
End Function

'========================================================================================================
'
' GetPictureInfo
'
' This function takes a given picture and finds out all possible information about it and returns the
' results.
'
' Parameter:              Use:
' --------------------------------------------------
' hPicture                Handle to the picture to get the information from.
' Return_Height           Optional. Returns the height (in pixels) of the picture.
' Return_Width            Optional. Returns the width (in pixels) of the picture.
' Return_Type             Optional. Returns the picture type:
'                           IMAGE_BITMAP      = 0
'                           IMAGE_CURSOR      = 1
'                           IMAGE_ICON        = 2
'                           IMAGE_ENHMETAFILE = 3
' Return_BitsPerPixel     Optional. Returns the color depth of the picture in the form of "BitsPerPixel"
' Return_Size             Optional. Returns the approximate size of the picture (assuming it's RGB)
'
' Return:
' -------
' If the function succeeds, the return is TRUE
' If the function fails, the return is FALSE
'
'========================================================================================================
Public Function GetPictureInfo(ByVal hPicture As Long, _
                               Optional ByRef Return_Height As Long, _
                               Optional ByRef Return_Width As Long, _
                               Optional ByRef Return_Type As Long, _
                               Optional ByRef Return_BitsPerPixel As Integer, _
                               Optional ByRef Return_Size As Double) As Boolean
On Error Resume Next
  
  Dim BMP As BITMAP
  
  ' Clear the return variables
  Return_Height = 0
  Return_Width = 0
  Return_Type = -1
  Return_BitsPerPixel = 0
  Return_Size = 0
  
  ' Check that there's a valid input
  If hPicture = 0 Then Exit Function
  
  ' Get the information
  If GetObjectAPI(hPicture, Len(BMP), BMP) = 0 Then Exit Function
  
  ' Return the information
  With BMP
    Return_BitsPerPixel = .bmBitsPixel
    Return_Height = .bmHeight
    Return_Width = .bmWidth
    Return_Type = .bmType
    Return_Size = ((.bmWidth * 3 + 3) And &HFFFFFFFC) * .bmHeight
  End With
  
  ' Function succeeded
  GetPictureInfo = True
  
End Function

'========================================================================================================
'
' GetPictureFromDC
'
' This function returns the handle to the picture that is currently selected into the specified Device
' Context (DC).  This does not remove the picture from the DC, it just gives you the handle to the picture
' so you can check the picture's height, width, color depth, etc. if you wish.
'
' Parameter:              Use:
' --------------------------------------------------
' hDC                     Handle to the DC to get the picture from
'
' Return:
' -------
' If the function succeeds, the return is the handle to the DC's picture
' If the function fails, the return is ZERO (0)
'
'========================================================================================================
Public Function GetPictureFromDC(ByVal hDC As Long) As Long
  
  GetPictureFromDC = GetCurrentObject(hDC, OBJ_BITMAP)
  
End Function

'========================================================================================================
'
' MemoryDC_Create
'
' This function creates a Device Context (DC) in memory compatible with the current screen display.
' This DC can be used to hold or render Bitmaps, Brushes, Fonts, Pens, or Regions (see also : Win32 API
' documentation for the "SelectObject" function).
'
' IMPORTANT: When you are done with the DC created by this function, you must delete it by calling the
' MemoryDC_Delete function in this module, or the Win32 API "DeleteDC".  If you do not, you could cause
' a VERY LARGE memory leak in your program.
'
' Parameter:              Use:
' --------------------------------------------------
' None
'
' Return:
' -------
' If the function succeeds, the return is the handle to the newly created Device Context
' If the function fails, the return is ZERO (0)
'
'========================================================================================================
Public Function MemoryDC_Create() As Long
  
  Dim hScreenDC   As Long
  
  ' Get the handle to the current display screen's DC
  hScreenDC = GetDC(GetDesktopWindow)
  
  ' Create a compatible DC
  If hScreenDC <> 0 Then MemoryDC_Create = CreateCompatibleDC(hScreenDC)
  
  ' Release the handle to the display screen
  ReleaseDC GetDesktopWindow, hScreenDC
  
End Function

'========================================================================================================
'
' MemoryDC_Delete
'
' This function deletes any memory Device Context (DC) created by a call to such Win32 APIs as "CreateDC"
' or "CreateCompatibleDC".  Any DC created by such Win32 API calls *MUST* be deleted by calling this
' function or the "DeleteDC" Win32 API Function.  If you do not, you could cause a VERY LARGE memory
' leak in your program.
'
' Parameter:              Use:
' --------------------------------------------------
' hMemoryDC               Handle to the memory device context to delete
'
' Return:
' -------
' If the function succeeds, the return is TRUE
' If the function fails, the return is FALSE
'
'========================================================================================================
Public Function MemoryDC_Delete(ByRef hMemoryDC As Long, _
                                Optional ByRef hPrevBitmap As Long) As Boolean
  
  ' Make sure parameter(s) are valid
  If hMemoryDC = 0 Then Exit Function
  
  ' Put the previous bitmap back into the DC and delete the one that's in it now
  If hPrevBitmap <> 0 Then DeleteObject SelectObject(hMemoryDC, hPrevBitmap)
  
  ' Delete the DC
  If DeleteDC(hMemoryDC) <> 0 Then MemoryDC_Delete = True
  
  ' Clear the values
  hMemoryDC = 0
  hPrevBitmap = 0
  
End Function

'========================================================================================================
'
' LoadPictureToDC
'
' This function loads the specified picture into the specified Device Context (DC) and returns the handle
' to the picture that was previously in the specified DC.  This is important, because that picture
' should be reloaded back into the DC before deleting the DC.
'
' NOTE: You can call the "MemoryDC_Delete" function of this module and pass it the returned picture
' handle and it will properly restore the old picture, delete the new picture, then delete the DC.
'
' Parameter:              Use:
' --------------------------------------------------
' hDC                     Handle to the DC to load the picture into
' hPictureToLoad          Handle to the picture to load into the DC
' Return_hPrevPic         Optional. Returns the handle to the picture that was previously in the DC
'
' Return:
' -------
' If the function succeeds, the return is TRUE
' If the function fails, the return is FALSE
'
'========================================================================================================
Public Function LoadPictureToDC(ByVal hDC As Long, _
                                ByVal hPictureToLoad As Long, _
                                Optional ByRef Return_hPrevPic As Long) As Boolean
   
  ' Make sure the values passed are valid
  If hDC = 0 Or hPictureToLoad = 0 Then Exit Function
  
  ' Select the specified picture into the specified DC
  Return_hPrevPic = SelectObject(hDC, hPictureToLoad)
  
  ' Return successful results
  LoadPictureToDC = True
  
End Function

'========================================================================================================
'
' RefreshHWND
'
' This function "refreshes" the specified window by invalidating it's contents.  This sends messages
' to the specified window that it needs to repaint itself.
'
' NOTE - If a picture is drawn on a VB Form, PictureBox, etc and that object's "AutoRedraw" property
' is set to TRUE, the picture will not immediately appear.  You must call this function, or the object's
' "Refresh" method to see the newly drawn picture on it.  However, if that object's "AutoRedraw" property
' is set to FALSE, the picture will immediately appear and any call to this function or that object's
' "Refresh" method will cause the picture to be erased.
'
' NOTE - Pictures are drawn to the "Image" property of a VB object, not the "Picture" property.  You
' can set the "Picture" property like this :  Set Picture1.Picture = Picture1.Image
'
' Parameter:              Use:
' --------------------------------------------------
' WindowHandle            Handle to the window or object to refresh
'
' Return:
' -------
' If the function succeeds, the return is TRUE
' If the function fails, the return is FALSE
'
'========================================================================================================
Public Function RefreshHWND(ByVal WindowHandle As Long) As Boolean
On Error Resume Next
  
' If RedrawWindow(WindowHandle, ByVal 0, 0, RDW_INVALIDATE Or RDW_ALLCHILDREN) <> 0 Then RefreshHWND = True
  If RedrawWindow(WindowHandle, ByVal 0, 0, RDW_INVALIDATE) <> 0 Then RefreshHWND = True
  
End Function

'========================================================================================================
'
' RenderPicture
'
' This function takes the handle to a Picture (ie - PictureBox1.Picture.Handle) or the handle to a Device
' Context (DC) and renders (or paints) it onto the specified output DC (ie - PictureBox2.hDC).  This
' function also gives you the ability to stretch the picture to a specified height/width before rendering
' it to the output DC.  As an aditional option, you can refresh a specified window that is related to
' the specified DC (ie - PictureBox2.hWnd).
'
' NOTE : All measurements for this function are expected to be PIXELS.  The default measurement of
'        pictures in Visual Basic is HIMETRIC.  Convert the dimentions to pixels if need be using the
'        "ConvPicDimentions" function before calling this function, or use the "GetPictureInfo" function
'        to get the height and width in pixels.
'
' NOTE : If you specify a form as the source hDC and do NOT specify a picture, you must specify the
'        picture's height/width.  Otherwise, the image rendered will have the same height and width as
'        your current screen resolution (800x600, 1024x768, etc)
'
' NOTE : This function is the most efficient if you specify a handle to a DC in the Source_hDC parameter
'        that contains the picture specified in the hPicture parameter as well as the height and width
'        of the picture specified.  That way no DC's have to be created or queried.
'
' Parameter:              Use:
' --------------------------------------------------
' Dest_hDC                The DC that the image is to be drawn onto
' Source_hDC              Optional. If this parameter is specified and the hPicture parameter is not,
'                         this function takes the currently selected bitmap from the DC and uses it
' hPicture                Optional. If this parameter is specified, a memory DC is created and the
'                         picture is selected into it to be used to paint the picture to the specified
'                         DC.  If this parameter is NOT specified, there must be a valid DC in the
'                         Source_DC parameter containing a valid picture to use.
' Dest_X                  Optional. Specifies X (Left) coordinate of where the picture should be rendered
'                         on the specified Dest_hDC (pixels).
' Dest_Y                  Optional. Specifies Y (Top) coordinate of where the picture should be rendered
'                         on the specified Dest_hDC (pixels).
' Src_X                   Optional. Specifies X (Left) coordinate of where the picture should be taken
'                         from on the specified Source_hDC (pixels).
' Src_Y                   Optional. Specifies Y (Top) coordinate of where the picture should be taken
'                         from on the specified Source_hDC (pixels).
' PicHeight               Optional. This specifies the height (in pixels) of the picture to be rendered.
'                         If this parameter is NOT specified, the function attempts to find out the
'                         height of the picture and use that.
' PicWidth                Optional. This specifies the width (in pixels) of the picture to be rendered.
'                         If this parameter is NOT specified, the function attempts to find out the
'                         width of the picture and use that.
' RasterOperation         Optional. This specifies the raster operation to be used in painting the
'                         picture.  The default is SRCCOPY (which just copies the picture).
' StretchPicture          Optional. If this parameter is set to TRUE, the picture is stretched to the
'                         size specified in the StretchHeight and StretchWidth parameters.
' StretchHeight           Optional. If the StretchPicture parameter is set to TRUE, this parameter
'                         specifies the height to stretch the picture to.
' StretchWidth            Optional. If the StretchPicture parameter is set to TRUE, this parameter
'                         specifies the width to stretch the picture to.
' RefreshWindow           Optional. If this parameter is set to TRUE, the function refreshes the window
'                         specified in the RefreshHandle parameter (which should be the handle of the
'                         window associated with the DC specified in the Dest_hDC parameter.
' RefreshHandle           Optional. If the RefreshWindow parameter is set to TRUE, this parameter
'                         specifies the handle of the window to refresh.
'
' Return:
' -------
' If the function succeeds, the return is TRUE
' If the function fails, the return is FALSE
'
'========================================================================================================
Public Function RenderPicture(ByVal Dest_hDC As Long, _
                              Optional ByVal Source_hDC As Long, _
                              Optional ByVal hPicture As Long, _
                              Optional ByVal Dest_X As Long, _
                              Optional ByVal Dest_Y As Long, _
                              Optional ByVal Src_X As Long, _
                              Optional ByVal Src_Y As Long, _
                              Optional ByVal PicHeight As Long, _
                              Optional ByVal PicWidth As Long, _
                              Optional ByVal RasterOperation As RasterOperations = SRCCOPY, _
                              Optional ByVal StretchPicture As Boolean = False, _
                              Optional ByVal StretchHeight As Long, _
                              Optional ByVal StretchWidth As Long, _
                              Optional ByVal RefreshWindow As Boolean = False, _
                              Optional ByVal RefreshHandle As Long) As Boolean
On Error Resume Next
  
  Dim ScrHWND    As Long
  Dim ScrHDC     As Long
  Dim hMemoryDC  As Long
  Dim BMP        As BITMAP
  Dim bDelMemDC  As Boolean
  Dim hOldBitmap As Long
  
  ' Make sure parameters passed are valid
  If Dest_hDC = 0 Then
    Exit Function
  ElseIf Source_hDC = 0 And hPicture = 0 Then
    Exit Function
  End If
  
  ' If no source DC was specified, but a picture was specified, create a DC to use
  If Source_hDC = 0 And hPicture <> 0 Then
    bDelMemDC = True
    ScrHWND = GetDesktopWindow
    ScrHDC = GetDC(ScrHWND)
    hMemoryDC = CreateCompatibleDC(ScrHDC)
    ReleaseDC ScrHWND, ScrHDC
    If hMemoryDC = 0 Then GoTo CleanUp
    hOldBitmap = SelectObject(hMemoryDC, hPicture)
    
  ' If a source DC was specified and no picture was specified, make sure the source hDC has a picture
  ElseIf Source_hDC <> 0 And hPicture = 0 Then
    bDelMemDC = False
    hMemoryDC = Source_hDC
    hPicture = GetPictureFromDC(Source_hDC)
    If hPicture = 0 Then
      Exit Function
    End If
    
  ' If a source DC AND a picture was specified, use them both
  ElseIf Source_hDC <> 0 And hPicture <> 0 Then
    bDelMemDC = False
    hMemoryDC = Source_hDC
  End If
  
  ' If the user didn't specify a Height / Width, get them from the picture
  If PicHeight = 0 Or PicWidth = 0 Then
    If GetObjectAPI(hPicture, Len(BMP), BMP) = 0 Then GoTo CleanUp
    If PicHeight = 0 Then PicHeight = BMP.bmHeight
    If PicWidth = 0 Then PicWidth = BMP.bmWidth
  End If
  
  ' Check if the user wants to stretch the picture
  If StretchPicture = True Then
    If StretchHeight = PicHeight And StretchWidth = PicWidth Then
      StretchPicture = False
    ElseIf StretchHeight = 0 Or StretchWidth = 0 Then
      StretchPicture = False
    End If
  End If
  
  ' Render the picture onto the specified DC
  If StretchPicture = False Then
    If BitBlt(Dest_hDC, Dest_X, Dest_Y, PicWidth, PicHeight, hMemoryDC, Src_X, Src_Y, RasterOperation) <> 0 Then
      RenderPicture = True
    End If
  Else
    If StretchBlt(Dest_hDC, Dest_X, Dest_Y, StretchWidth, StretchHeight, hMemoryDC, Src_X, Src_Y, PicWidth, PicHeight, RasterOperation) <> 0 Then
      RenderPicture = True
    End If
  End If
  
  ' Refresh the DC to show the picture just drawn on it
  If RenderPicture = True And RefreshWindow = True And RefreshHandle <> 0 Then
    RefreshHWND RefreshHandle
  End If
  
CleanUp:
  
  If bDelMemDC = True And hMemoryDC <> 0 Then
    SelectObject hMemoryDC, hOldBitmap
    DeleteDC hMemoryDC
  End If
  
End Function

'========================================================================================================
'
' TilePicture
'
' This function makes it easy to tile the specified picture onto any Device Context (DC).
'
' 
' Dim PicImg As StdPicture
'
' Me.Show
' Me.ScaleMode = vbPixels
'
' Set PicImg = LoadPicture("C:\Image.bmp")
' TilePicture PicImg.Handle, Me.hDC, Me.ScaleWidth, Me.ScaleHeight, True, Me.hWnd
' Set PicImg = Nothing
'
' 
'
' Parameter:              Use:
' --------------------------------------------------
' hPicture                Specifies the handle to the image to tile
' Dest_hDC                Specifies the Device Context to tile to
' MaxWidth                Specifies the maximum width to tile to
' MaxHeight               Specifies the maximum height to tile to
' RefreshWindow           Optional. If set to TRUE, the window specified by "RefreshHandle" will be refreshed
' RefreshHandle           Optional. If the "RefreshWindow" property is set to true, this specifies the
'                         handle to the window to be refreshed
'
' Return:
' -------
' If the function succeeds, the return is TRUE
' If the function fails, the return is FALSE
'
'========================================================================================================
Public Function TilePicture(ByVal hPicture As Long, _
                            ByVal Dest_hDC As Long, _
                            ByVal MaxWidth As Long, _
                            ByVal MaxHeight As Long, _
                            Optional ByVal RefreshWindow As Boolean, _
                            Optional ByVal RefreshHandle As Long) As Boolean
  
  Dim TheHeight As Long
  Dim TheWidth  As Long
  Dim CounterX  As Long
  Dim CounterY  As Long
  Dim MaxX      As Long
  Dim MaxY      As Long
  Dim hTempDC   As Long
  Dim hPrevBMP  As Long
  
  ' Make sure parameters are valid
  If hPicture = 0 Or Dest_hDC = 0 Or MaxWidth = 0 Or MaxHeight = 0 Then Exit Function
  
  ' Get the picture's dimentions
  If GetPictureInfo(hPicture, TheHeight, TheWidth) = False Then Exit Function
  
  ' Create a temporary DC to do BitBlt'ing with
  hTempDC = MemoryDC_Create
  hPrevBMP = SelectObject(hTempDC, hPicture)
  
  ' Get how many times to loop to cover the specified area
  If MaxWidth <= TheWidth Then
    MaxX = 1
  Else
    MaxX = MaxWidth / TheWidth
    If MaxWidth Mod TheWidth <> 0 Then MaxX = MaxX + 1
  End If
  If MaxHeight <= TheHeight Then
    MaxY = 1
  Else
    MaxY = MaxHeight / TheHeight
    If MaxHeight Mod TheHeight <> 0 Then MaxY = MaxY + 1
  End If
  
  ' If the MaxWidth and MaxHeight are less than the dimetions of the picture, just draw it once and exit
  If MaxX = 1 And MaxY = 1 Then
    TilePicture = RenderPicture(Dest_hDC, hTempDC, hPicture, CounterX, CounterY, 0, 0, TheHeight, TheWidth, , , , , RefreshWindow, RefreshHandle)
    TilePicture = True
    GoTo CleanUp
  End If
  
  ' Loop while the tile isn't complete
  For CounterX = 0 To (TheWidth * MaxX) Step TheWidth
    For CounterY = 0 To (TheHeight * MaxY) Step TheHeight
      If BitBlt(Dest_hDC, CounterX, CounterY, TheWidth, TheHeight, hTempDC, 0, 0, SRCCOPY) = 0 Then GoTo CleanUp
      'If RenderPicture(Dest_hDC, hTempDC, hPicture, CounterX, CounterY, 0, 0, TheHeight, TheWidth, , , , , RefreshWindow, RefreshHandle) = False Then GoTo CleanUp
    Next CounterY
  Next CounterX
  
  ' Return Succeeded
  TilePicture = True
  
CleanUp:
  
  ' Clean up memory
  DeleteObject SelectObject(hTempDC, hPrevBMP)
  DeleteDC hTempDC
  DeleteObject hPrevBMP
  hTempDC = 0
  hPrevBMP = 0
  
End Function

' Function that converts automation colors such as "vbButtonFace" to standard
' color such as "12632256".  It is safest to pass all colors through this
' function to make sure that if a user passes a color like "Me.BackColor" and
' the BackColor is vbButtonFace, it won't mess up any of the API's that are
' expecting a normal color value.
Public Function TranslateColor(ByVal oClr As Long, _
                               Optional ByVal hPal As Long = 0) As Long
On Error Resume Next
  
  If OleTranslateColor(oClr, hPal, TranslateColor) <> 0 Then TranslateColor = -1
  
End Function
